module Graphics.Rendering.Hieroglyph.Colors where

import Graphics.Rendering.Hieroglyph.Cairo

strokeWhite = StrokeRGBA 1 1 1 1
strokeBlack = StrokeRGBA 0 0 0 1
strokeRed = StrokeRGBA 1 0 0 1
strokeGreen = StrokeRGBA 0 1 0 1
strokeBlue = StrokeRGBA 0 0 1 1
strokeYellow = StrokeRGBA 1 1 0 1
strokeViolet = StrokeRGBA 1 0 1 1
strokeCyan = StrokeRGBA 0 1 1 1
strokeDarkRed = attenuate 0.5 strokeRed
strokeDarkGreen = attenuate 0.5 strokeGreen
strokeDarkBlue = attenuate 0.5 strokeBlue
strokeDarkYellow = attenuate 0.5 strokeYellow 
strokeDarkViolet = attenuate 0.5 strokeViolet
strokeDarkCyan = attenuate 0.5 strokeCyan

strokeGrey90 = attenuate 0.9 strokeWhite
strokeGrey80 = attenuate 0.8 strokeWhite
strokeGrey70 = attenuate 0.7 strokeWhite
strokeGrey60 = attenuate 0.6 strokeWhite
strokeGrey50 = attenuate 0.5 strokeWhite
strokeGrey40 = attenuate 0.4 strokeWhite
strokeGrey30 = attenuate 0.3 strokeWhite
strokeGrey20 = attenuate 0.2 strokeWhite
strokeGrey10 = attenuate 0.1 strokeWhite

fillWhite = FillRGBA 1 1 1 1
fillBlack = FillRGBA 0 0 0 1
fillRed = FillRGBA 1 0 0 1
fillGreen = FillRGBA 0 1 0 1
fillBlue = FillRGBA 0 0 1 1
fillYellow = FillRGBA 1 1 0 1
fillViolet = FillRGBA 1 0 1 1
fillCyan = FillRGBA 0 1 1 1
fillDarkRed = attenuate 0.5 fillRed
fillDarkGreen = attenuate 0.5 fillGreen
fillDarkBlue = attenuate 0.5 fillBlue
fillDarkYellow = attenuate 0.5 fillYellow 
fillDarkViolet = attenuate 0.5 fillViolet
fillDarkCyan = attenuate 0.5 fillCyan

fillGrey90 = attenuate 0.9 fillWhite
fillGrey80 = attenuate 0.8 fillWhite
fillGrey70 = attenuate 0.7 fillWhite
fillGrey60 = attenuate 0.6 fillWhite
fillGrey50 = attenuate 0.5 fillWhite
fillGrey40 = attenuate 0.4 fillWhite
fillGrey30 = attenuate 0.3 fillWhite
fillGrey20 = attenuate 0.2 fillWhite
fillGrey10 = attenuate 0.1 fillWhite

toStroke (FillRGBA r g b a) = (StrokeRGBA r g b a)
toStroke (StrokeRGBA r g b a) = (StrokeRGBA r g b a)
toFill (StrokeRGBA r g b a) = (FillRGBA r g b a)
toFill (FillRGBA r g b a) = (FillRGBA r g b a)

unattenuate by (FillRGBA r g b a) = FillRGBA (clamp 0 1 $ (1+by)*r) (clamp 0 1 $ (1+by)*g) (clamp 0 1 $ (1+by)*b) a
unattenuate by (StrokeRGBA r g b a) = StrokeRGBA (clamp 0 1 $ (1+by)*r) (clamp 0 1 $ (1+by)*g) (clamp 0 1 $ (1+by)*b) a
attenuate by (FillRGBA r g b a) = FillRGBA (clamp 0 1 $ by*r) (clamp 0 1 $ by*g) (clamp 0 1 $ by*b) a
attenuate by (StrokeRGBA r g b a) = StrokeRGBA (clamp 0 1 $ by*r) (clamp 0 1 $ by*g) (clamp 0 1 $ by*b) a
fade by (FillRGBA r g b a) = FillRGBA r g b (clamp 0 1 $ by*a)
fade by (StrokeRGBA r g b a) = StrokeRGBA r g b (clamp 0 1 $ by*a)
unfade by (FillRGBA r g b a) = FillRGBA r g b (clamp 0 1 $ (1+by)*a)
unfade by (StrokeRGBA r g b a) = StrokeRGBA r g b (clamp 0 1 $ (1+by)*a)

mix ratio (FillRGBA r1 g1 b1 a1) (FillRGBA r2 g2 b2 a2) = (FillRGBA r' g' b' a') 
    where r' = clamp 0 1 $ (r1+r2) * ratio
          g' = clamp 0 1 $ (g1+g2) * ratio
          b' = clamp 0 1 $ (b1+b2) * ratio
          a' = clamp 0 1 $ (a1+a2) * ratio

clamp lo hi val = min hi (max lo val)


